const fs = require('fs');
const fsPromises = require('fs/promises');
const { exec } = require("child_process");
const { exit } = require('process');
const fetch = require('node-fetch');
const JSZip = require('jszip');
const puppeteer = require('puppeteer');

process.env.NODE_TLS_REJECT_UNAUTHORIZED = "0";
const apiURL = "http://127.0.0.1/api";
const statexplorerUrl = "http://localhost/statexplorer";


async function getPageContentsWithPuppeteer(url, type) {
  var browser = null;
  try {
    console.log(`Launching browser`);
    browser = await puppeteer.launch({
      args: ["--no-sandbox"],
      headless: true
    });
    console.log(`Creating page`);
    const page = await browser.newPage();

    console.log(`Loading initial page`);
    await page.goto(url);

    console.log(`Loading results`);
    await page.waitForSelector('#export-finished');

    console.log(`Converting to ${type}`);
    var contents = [];
    switch (type) {
      case "pdf":
        contents.push(await page.pdf({
          displayHeaderFooter: false,
          printBackground: true,
          format: 'A4',
          landscape: true
        }));
        break;
      case "csv":
        contents = await page.evaluate(() => {
          return Promise.resolve(window.csvs);
        });
        break;
      case "html":
      default:
        contents.push(await page.content());
        break;
    }

    console.log(`Closing browser`);
    await browser.close();
    return contents;
  }
  catch (ex) {
    console.error(e);
    if (browser) {
      await driver.close();
    }
    exit(1);
  }
}


const args = process.argv.slice(2);
if (args.length !== 9) {
  console.log('Wrong arguments !');
  console.log('');
  console.log('Usage: ');
  console.log('node pdfgenerator.js distro languagecode plugins userid filter email filetype');
  console.log('  distro: centos/debian');
  console.log('  languagecode: en_US/fr_FR/es_ES');
  console.log('  plugins: list of plugins to get data, in the form of : "pluginname1,pluginname2,..."');
  console.log('  userid: Numerical user ID of the user that should generate the data (exemple 1)');
  console.log('  filter: Name of filter to apply (use " if space inside)');
  console.log('  email: Name of recipient to send the result');
  console.log('  filetype: pdf/html/csv');
  console.log('  subject: subject of the mail');
  console.log('  body: content of the mail');
  console.log('');
  console.log('Example: node pdfgenerator.js debian en "hourrepartition,calllists" 1 "Last month" o.varenne@ipconnect.fr pdf "my subject" "the content of my email"');
  exit(1);
}

async function getApiToken() {
  return new Promise(async (resolve, reject) => {
    try {
      var body = new URLSearchParams();
      body.append('key', '2c86b708-35ea-48b5-8162-53e15d961293');

      //if https, we should add an option like:
      //agent: new https.Agent({ rejectUnauthorized: false })
      //but if it's http it crash as it does not understand why we talk about https... so the redirect is a problem.

      var res = await fetch(`${apiURL}/authenticate`, {
        method: 'POST',
        body: body
      });
      if (!res.ok) {
        console.error("Unable to get token");
        reject(new Error('request failed'));
        return;
      }
      var json = await res.json();
      if (json.status === "success") {
        resolve(json.data.token);
        return;
      }
      console.error(json.message);
      reject(new Error(json.status));
    }
    catch (ex) {
      reject(ex);
    }

  });
}

async function getMailFromAsync() {
  return new Promise(async (resolve, reject) => {
    try {
      var token = await getApiToken();
      console.log(`Got api token ${token}`);
      var body = new URLSearchParams();
      body.append('token', token);
      body.append('module', 'system_misc');
      body.append('name', 'fromemail');
      //if https, we should add an option like:
      //agent: new https.Agent({ rejectUnauthorized: false })
      //but if it's http it crash as it does not understand why we talk about https... so the redirect is a problem.

      var res = await fetch(`${apiURL}/get_setting`, {
        method: 'POST',
        body: body
      });
      if (!res.ok) {
        console.error("Unable to get mail from");
        reject(new Error('request failed'));
        return;
      }
      var json = await res.json();
      if (json.status === "success") {
        resolve(json.data);
        return;
      }
      console.error(json.message);
      reject(new Error(json.status));
    }
    catch (ex) {
      reject(ex);
    }
  });
}


async function storeAttachements(contents) {
  var filesGenerated = [];
  switch (type) {
    case "csv":
      for (var cont of contents) {
        var filename = `/tmp/export_${cont.name}.csv`;
        console.log(`Creating attachement ${filename}`);
        filesGenerated.push(filename);
        await fsPromises.writeFile(filename, cont.content);
      }
      break;
    case "html":
    case "pdf":
    default:
      for (var i = 0; i < contents.length; i++) {
        var filename = `/tmp/export_${i}.${type}`;
        console.log(`Creating attachement ${filename}`);
        filesGenerated.push(filename);
        await fsPromises.writeFile(filename, contents[i]);
      }
      break;
  }
  return filesGenerated;
}



async function buildMailCommand(distro, filesGenerated) {
  var mailfrom = "";
  try {
    mailfrom = await getMailFromAsync();
    console.log(`Got mailfrom ${mailfrom}`);
  }
  catch (ex) {
    console.error("Unable to get mail from. Using default.", JSON.stringify(ex));
  }

  var mailfromoption = "";
  var mailcommand = "";
  switch (distro) {
    case "debian":
      mailfromoption = mailfrom !== "" ? `-a "from=${mailfrom}" -r "${mailfrom}"` : "";
      mailcommand = `echo "${body}" | mail ${mailfromoption} -A "${filesGenerated.join('" -A "')}" -s "${subject}" ${emails.replace(/,/g, " ").replace(/;/g, " ")}`;
      break;
    default:
      mailfromoption = mailfrom !== "" ? `-S "from=${mailfrom}" -r "${mailfrom}"` : "";
      mailcommand = `echo "${body}" | mailx ${mailfromoption} -a "${filesGenerated.join('" -a "')}" -s "${subject}" ${emails.replace(/,/g, " ").replace(/;/g, " ")}`;
      break;
  }
  return mailcommand;
}


async function exportAndMail(distro, url, emails, type, subject, body) {
  try {
    //const filename = await capture(url, type, true);
    const contents = await getPageContentsWithPuppeteer(url, type);
    const filesGenerated = await storeAttachements(contents);
    const mailcommand = await buildMailCommand(distro, filesGenerated);

    console.log(mailcommand);
    exec(mailcommand, (error, stdout, stderr) => {
      for (var f of filesGenerated) {
        fs.unlinkSync(f);
      }
      if (error) {
        console.log("Done, but failed to send file");
        console.log(stdout, stderr);
        exit(1);
      }
      console.log("Done and sent to " + emails);
      exit(0);
    });
  }
  catch (e) {
    console.error(e);
    exit(1);
  }
}



// START OF SCRIPT !

const distro = args[0];
const lang = args[1];
const plugins = JSON.stringify(args[2].split(','));
const userid = args[3];
const filter = args[4];
const emails = args[5];
const type = args[6];
const subject = args[7];
const body = args[8];

const url = `${statexplorerUrl}/exporttopdf.php?lang=${lang}&plugins=${plugins}&userid=${userid}&filter=${filter}`;
exportAndMail(distro, url, emails, type, subject, body);